////////////////////////////////////////////////////////////////
//                                               //
////////////////////////////////////////////////////////////////

unit UOpenGL;

interface

uses Windows, OpenGL;

type

     TOpenGL = class       //  3D   OpenGL
     private
       FWnd      :hWND;                              // ID  
       FDC       :hDC;                               //  
       FRC       :hGLRC;                             //  
       FAngleX   :double;                            //    X
       FAngleY   :double;                            //    Y
       FAngleZ   :double;                            //    Z
       FN1       :longint;                           //    
       FN2       :longint;                           //    
       FN3       :longint;                           //    
       FH        :double;                            // 
       FMS       :double;                            // 
       function Xe(X:longint):double;                //   ->  
       function Ye(Y:longint):double;                //   ->  
       function Ze(Z:longint):double;                //   ->  
       procedure SetDCPixelFormat(DC: hDC);          //     DC
       procedure Init(Wnd: hWND);                    //  OpenGL
       procedure Kill;                               //    OpenGL
       procedure SetWnd(Wnd:hWND);                   //  Wnd
     public
       constructor Create;                           //  
       destructor Destroy; override;                 //  
       procedure Draw;                               // 
       procedure Resize(Width, Height: Integer);     //  
       procedure Run;                                // - 
       property Window:hWND read FWnd write SetWnd;  // ID  
       property N1:longint read FN1 write FN1;       //    
       property N2:longint read FN2 write FN2;       //    
       property N3:longint read FN3 write FN3;       //    
       property H :double  read FH  write FH;        // 
       property AngleX:double read FAngleX write FAngleX;  //    X
       property AngleY:double read FAngleY write FAngleY;  //    Y
       property AngleZ:double read FAngleZ write FAngleZ;  //    Z
       property MS:double  read FMS write FMS;       // 
     end;

var  GL3D :TOpenGL;   // 3D 

implementation

////////////////////////////////////////////////////////////////

function TOpenGL.Xe(X:longint):double;  //   ->  
begin
     result:=(X-(n1/2))*H;
end;

function TOpenGL.Ye(Y:longint):double;  //   ->  
begin
     result:=(Y-(n2/2))*H;
end;

function TOpenGL.Ze(Z:longint):double;  //   ->  
begin
     result:=(Z-(n3/2))*H;
end;

procedure TOpenGL.SetDCPixelFormat(DC: hDC);  //     DC
var  pfd          :TPixelFormatDescriptor;
     nPixelFormat :Integer;
begin
     FillChar(pfd, SizeOf(pfd), 0);
     with pfd do begin
          nSize      := sizeof(pfd);
          nVersion   := 1;
          dwFlags    := PFD_DRAW_TO_WINDOW or
                        PFD_SUPPORT_OPENGL or
                        PFD_DOUBLEBUFFER;
          iPixelType := PFD_TYPE_RGBA;
          cColorBits := 16;
          cDepthBits := 64;
          iLayerType := PFD_MAIN_PLANE;
     end;
     nPixelFormat := ChoosePixelFormat(DC, @pfd);
     SetPixelFormat(DC, nPixelFormat, @pfd);
end;

procedure TOpenGL.Init(Wnd: hWND);  //  OpenGL
const
      // ,    
      mat1_amb : array [0..2] of Single = (0.2, 0.2, 0.2);
      mat1_dif : array [0..2] of Single = (0.8, 0.8, 0.0);
      mat1_spec: array [0..2] of Single = (0.6, 0.6, 0.6);
      mat1_shininess = 10;
      //    
      light_pos  : array [0..3] of glFloat = (100.0, 100.0, 0.0, 1.0);
      light_amb  : array [0..3] of glFloat = (0.6, 0.6, 0.6, 1.0);
      light_dif  : array [0..3] of glFloat = (1.0, 1.0, 1.0, 1.0);
      light_spec : array [0..3] of glFloat = (1.0, 1.0, 1.0, 1.0);
      light_spot_direction : array [0..3] of glFloat = (1.0, 1.0, 1.0, 1.0);
      //  
      fogColor: array [0..3] of GLfloat = (0, 1.0, 0, 1.0);
var   fogMode: GLint;
begin
     //    OpenGL
     FDC := GetDC(Wnd);             //     
     SetDCPixelFormat(FDC);         //   
     FRC := wglCreateContext(FDC);  //    
     wglMakeCurrent(FDC,FRC);       //   
     //     glEnable(GL_COLOR_MATERIAL)
     glMaterialfv(GL_FRONT, GL_AMBIENT,  @mat1_amb);
     glMaterialfv(GL_FRONT, GL_DIFFUSE,  @mat1_dif);
     glMaterialfv(GL_FRONT, GL_SPECULAR, @mat1_spec);
     glMaterialf (GL_FRONT, GL_SHININESS,mat1_shininess);
     //    GL_LIGHT1   glEnable(GL_LIGHTING)
     //    :  glEnable(GL_LIGHT0);
     glLightfv(GL_LIGHT1, GL_POSITION,@light_pos);
     glLightfv(GL_LIGHT1, GL_AMBIENT, @light_amb);    // 
     glLightfv(GL_LIGHT1, GL_DIFFUSE, @light_dif);    // 
     glLightfv(GL_LIGHT1, GL_SPECULAR, @light_spec);
     glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, @light_spot_direction);
     //    GL_LIGHT1
     glEnable(GL_LIGHT1);
     //     glEnable(GL_FOG)
     fogMode := GL_EXP;                     // GL_EXP2, GL_LINEAR
     glFogi(GL_FOG_MODE, fogMode);          //  
     glFogfv(GL_FOG_COLOR, @fogColor);      //  
     glFogf(GL_FOG_DENSITY, 0.002);         //   - 0.2%
     glHint(GL_FOG_HINT, GL_DONT_CARE);     // GL_NICEST, GL_FASTEST.
     glClearColor(1.0, 1.0, 1.0, 1.0);      //    - 
     glClearDepth(1.0);                     //   
     glDepthFunc(GL_LESS);                  //   
     glShadeModel(GL_SMOOTH);               //   
     // glEnable(..) -  , glDisable(..) - 
     glEnable(GL_LIGHTING);                 //  
     //glEnable(GL_FOG);                    //  
     glEnable(GL_DEPTH_TEST);               //   ,  
                                            //    glDepthFunc
     glEnable(GL_NORMALIZE);                //   ( 
                                            //     )
     glEnable(GL_COLOR_MATERIAL);           //     
     glEnable(GL_TEXTURE_2D);               //   
     //glEnable(GL_BLEND);                  //   (. )
                                            //    glEnable(GL_DEPTH_TEST)
end;

procedure TOpenGL.Kill;  //    OpenGL
begin
     wglMakeCurrent(FDC, 0);
     wglDeleteContext(FRC);
     ReleaseDC(FWnd, FDC);
end;

procedure TOpenGL.Draw;  // 
begin
     //     
     glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
     glMatrixMode(GL_MODELVIEW);            //    
     glLoadIdentity;                        //   

     // GL_POINTS -  
     // GL_LINES -  ( ;  )
     // GL_LINE_STRIP -  (  lineto)
     // GL_LINE_LOOP - 
     // GL_TRIANGLES - 
     // GL_TRIANGLE_STRIP -   
     // GL_TRIANGLE_FAN
     // GL_QUADS - 
     // GL_QUAD_STRIP -   
     // GL_POLYGON -  

     // 
     glRotate(FAngleX, 1.0, 0.0, 0.0);
     glRotate(FAngleY, 0.0, 1.0, 0.0);
     glRotate(FAngleZ, 0.0, 0.0, 1.0);

     // 
     glBegin(GL_LINES);
     glColor(0.0, 0.0, 1.0, 0.0);
     glVertex3F(Xe( 1),Ye( 1),Ze( 1)); glVertex3F(Xe(n1),Ye( 1),Ze( 1));  // 1- 
     glVertex3F(Xe(n1),Ye( 1),Ze( 1)); glVertex3F(Xe(n1),Ye(n2),Ze( 1));  // 2- 
     glVertex3F(Xe(n1),Ye(n2),Ze( 1)); glVertex3F(Xe( 1),Ye(n2),Ze( 1));  // 3- 
     glVertex3F(Xe( 1),Ye(n2),Ze( 1)); glVertex3F(Xe( 1),Ye( 1),Ze( 1));  // 4- 
     glVertex3F(Xe( 1),Ye( 1),Ze(n3)); glVertex3F(Xe(n1),Ye( 1),Ze(n3));  // 5- 
     glVertex3F(Xe(n1),Ye( 1),Ze(n3)); glVertex3F(Xe(n1),Ye(n2),Ze(n3));  // 6- 
     glVertex3F(Xe(n1),Ye(n2),Ze(n3)); glVertex3F(Xe( 1),Ye(n2),Ze(n3));  // 7- 
     glVertex3F(Xe( 1),Ye(n2),Ze(n3)); glVertex3F(Xe( 1),Ye( 1),Ze(n3));  // 8- 
     glVertex3F(Xe( 1),Ye( 1),Ze( 1)); glVertex3F(Xe( 1),Ye( 1),Ze(n3));  // 9- 
     glVertex3F(Xe(n1),Ye( 1),Ze( 1)); glVertex3F(Xe(n1),Ye( 1),Ze(n3));  // 10- 
     glVertex3F(Xe(n1),Ye(n2),Ze( 1)); glVertex3F(Xe(n1),Ye(n2),Ze(n3));  // 11- 
     glVertex3F(Xe( 1),Ye(n2),Ze( 1)); glVertex3F(Xe( 1),Ye(n2),Ze(n3));  // 12- 
     glEnd;

{
     //    
     glBegin(GL_POLYGON);
       glColor(0.0, 0.5, 0.0, 0.0);            //  1- 
       glVertex3F(-50, -sqrt(3)*50, 0);        // 1- 
       glColor(0.5, 0.0, 0.0, 0.0);            //  2- 
       glVertex3F(-100, -sqrt(3)*100, 0);      // 2- 
       glColor(0.0, 0.0, 0.5, 0.0);            //  3- 
       glVertex3F(-125, -sqrt(3)*100, 0);      // 3- 
     glEnd;
}

     //     
     SwapBuffers(FDC);
end;

procedure TOpenGL.SetWnd(Wnd:hWND);  //  Wnd
begin
     FWnd:=Wnd;
     Init(FWnd);
end;


////////////////////////////////////////////////////////////////


constructor TOpenGL.Create;  //  
begin
     FWnd    := 0;     //    
     FDC     := 0;     //  
     FRC     := 0;     //  
     FAngleX := 30;    //    X
     FAngleY := -30;   //    Y
     FAngleZ := 0;     //    Z
     FN1     := 10;    //    
     FN2     := 10;    //    
     FN3     := 10;    //    
     FH      := 10;    // 
     FMS     := 200;   // 
end;

destructor TOpenGL.Destroy;  //  
begin
     Kill;        //    OpenGL
end;

procedure TOpenGL.Resize(Width, Height: Integer);  //  
var  X1,X2 :double;    //    X
     Y1,Y2 :double;    //    Y
     Z1,Z2 :double;    //    Z

begin
     if (Height = 0) then Height:=1;    //    0
     glViewport(0, 0, Width, Height);   //      
     //   
     glMatrixMode(GL_PROJECTION);       //   
     glLoadIdentity();                  //   
     //    - 
     X1:=-MS;    X2:=MS;
     Y1:=-MS*Height/Width;    Y2:=MS*Height/Width;
     Z1:=-MS;    Z2:=MS;
     glOrtho(X1,X2,Y1,Y2,Z1,Z2);
     //   
     glMatrixMode(GL_MODELVIEW);        //   
     glLoadIdentity();                  //   
end;


procedure TOpenGL.Run;  // - 
begin
     Draw;
end;


initialization
    GL3D := TOpenGL.Create;  //  3D 
finalization
    GL3D.Free;               //  3D 
end.
