unit UnitMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, Menus, ImgList, ComCtrls, ToolWin, StdCtrls, ExtCtrls;

type
  TFormMain = class(TForm)
    Status: TStatusBar;
    ToolBar: TToolBar;
    tbDisp: TToolButton;
    ImageList: TImageList;
    MainMenu: TMainMenu;
    ActionList: TActionList;
    Open: TOpenDialog;
    Save: TSaveDialog;
    mFile: TMenuItem;
    mHelp: TMenuItem;
    ControlBox: TGroupBox;
    ObjectBox: TGroupBox;
    acNew: TAction;
    acOpen: TAction;
    acSave: TAction;
    tbNew: TToolButton;
    tbOpen: TToolButton;
    tbSave: TToolButton;
    mNew: TMenuItem;
    mOpen: TMenuItem;
    mSave: TMenuItem;
    mSep: TMenuItem;
    mExit: TMenuItem;
    mHelpAbout: TMenuItem;
    acRun: TAction;
    tbSpace: TToolButton;
    tbRun: TToolButton;
    Panel: TPanel;
    lXlab: TLabel;
    lX: TLabel;
    lYlab: TLabel;
    lY: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    lZlab: TLabel;
    lZ: TLabel;
    procedure FormActivate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure acNewExecute(Sender: TObject);
    procedure acOpenExecute(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
    procedure mExitClick(Sender: TObject);
    procedure mHelpAboutClick(Sender: TObject);
    procedure acRunExecute(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure PanelMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PanelMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PanelMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

uses UAbout, UOpenGL;

{$R *.dfm}

var
     X0      :longint = 0;      //  X  
     Y0      :longint = 0;      //  Y  
     fRotate :boolean = false;  //  

////////////////////////////////////////////////////////////////
//                                            //
////////////////////////////////////////////////////////////////

procedure TFormMain.FormActivate(Sender: TObject);  //  
begin
     //     
     Open.InitialDir:=ExtractFileDir(Application.ExeName);
     Save.InitialDir:=Open.InitialDir;
     GL3D.Window:=Panel.Handle;
     FormResize(Sender);
end;

procedure TFormMain.FormPaint(Sender: TObject);  //  
begin
     GL3D.Draw;
end;

procedure TFormMain.FormResize(Sender: TObject);  //  
begin
     GL3D.Resize(Panel.ClientWidth,Panel.ClientHeight);
     FormPaint(Sender);
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     //        
end;

procedure TFormMain.PanelMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);   //  
begin
     fRotate:=true;  X0:=X;  Y0:=Y;  GL3D.Draw;
end;

procedure TFormMain.PanelMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);  // 
begin
     if not fRotate then Exit;
     GL3D.AngleY:=GL3D.AngleY+Round((X-X0)*0.01) mod 360;
     GL3D.AngleX:=GL3D.AngleX+Round((Y-Y0)*0.01) mod 360;
     GL3D.Draw;
     lX.Caption:=FloatToStr(GL3D.AngleX);
     lY.Caption:=FloatToStr(GL3D.AngleY);
     lZ.Caption:=FloatToStr(GL3D.AngleZ);
end;

procedure TFormMain.PanelMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);  //  
begin
     fRotate:=false;  GL3D.Draw;
end;

procedure TFormMain.FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);  // 
begin
     GL3D.MS:=GL3D.MS/0.9;
     GL3D.Resize(Panel.Width,Panel.Height);
     FormPaint(Sender);
end;

procedure TFormMain.FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);  // 
begin
     GL3D.MS:=GL3D.MS*0.9;
     GL3D.Resize(Panel.Width,Panel.Height);
     FormPaint(Sender); 
end;


////////////////////////////////////////////////////////////////
//                                            //
////////////////////////////////////////////////////////////////

procedure TFormMain.acNewExecute(Sender: TObject);  // 
begin
     Status.Panels.Items[0].Text:='';
end;

procedure TFormMain.acOpenExecute(Sender: TObject);  // 
begin
     if not Open.Execute then Exit;
     Status.Panels.Items[0].Text:=Open.FileName;
end;

procedure TFormMain.acSaveExecute(Sender: TObject);  // 
var  bmp :TBitmap;
     dc  :hDC;
begin
     if not Save.Execute then Exit;
     Status.Panels.Items[0].Text:=Save.FileName;

     bmp := TBitmap.Create;
     bmp.PixelFormat := pf24bit;
     bmp.width := Panel.Width;
     bmp.height:= Panel.Height;
     dc:=getdc(Panel.Handle);
     BitBlt(bmp.Canvas.Handle, 0, 0, bmp.Width, bmp.Height, dc, 0, 0, SRCCOPY);
     ReleaseDC(Panel.Handle, dc);
     bmp.SaveToFile(Save.FileName);
     bmp.free;
end;

procedure TFormMain.mExitClick(Sender: TObject);  // 
begin
     Close;
end;

procedure TFormMain.mHelpAboutClick(Sender: TObject);  //  
begin
     FormAbout.Left:=(FormMain.Width-FormAbout.Width) div 2;
     FormAbout.Top:=(FormMain.Height-FormAbout.Height) div 2;
     FormAbout.ShowModal;
end;

procedure TFormMain.acRunExecute(Sender: TObject);  // 
begin
     GL3D.Run;  // 
end;



end.
